import re
from pathlib import Path


def parse_encoder_log(encoder_log: str):
    log = ""
    if encoder_log and Path(encoder_log).is_file():
        with open(encoder_log, "r") as f:
            log = f.read()
    m = re.search(
        r".*?(x264 \[info]: frame I:.*?)\n"
        r".*?(x264 \[info]: frame P:.*?)\n"
        r".*?(x264 \[info]: frame B:.*?)\n"
        r".*?(x264 \[info]: consecutive B-frames:.*?)\n",
        log,
    )
    if m:
        return "\n".join(m.groups())
    m = re.search(
        r".*?(x265 \[info]: frame I:.*?)\n"
        r".*?(x265 \[info]: frame P:.*?)\n"
        r".*?(x265 \[info]: frame B:.*?)\n"
        r".*?(x265 \[info]: Weighted P-Frames:.*?)\n"
        r".*?(x265 \[info]: Weighted B-Frames:.*?)\n"
        r".*?(x265 \[info]: consecutive B-frames:.*?)\n",
        log,
    )
    if m:
        return "\n".join(m.groups())
    return ""



import requests
import webbrowser

from loguru import logger
from differential.constants import URL_SHORTENER_PATH


def open_link(link: str, use_short_url: bool = False):
    if use_short_url:
        req = requests.post(f"{URL_SHORTENER_PATH}/new", {"url": link})
        if req.ok:
            link = f"{URL_SHORTENER_PATH}/dft/{req.text}"

    try:
        browser = webbrowser.get()
    except webbrowser.Error:
        browser = None

    if browser is None or isinstance(browser, webbrowser.GenericBrowser):
        logger.info(f"未找到浏览器，请直接复制以下链接：{link}")
    else:
        browser.open(link, new=1)



import argparse

from differential.plugins.base import Base


class Unit3D(Base):
    @classmethod
    def get_aliases(cls):
        return "u3d",

    @classmethod
    def get_help(cls):
        return "Unit3D插件，适用于未经过大规模结构改动的Unit3D站点"

    @classmethod
    def add_parser(cls, parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
        super().add_parser(parser)
        return parser



import sys
import inspect
import pkgutil


class Wrapper:
    def __getattr__(self, item):
        return globals().get(item, None)


__all__ = []
for loader, name, is_pkg in pkgutil.walk_packages(__path__):
    m = loader.find_module(name)
    if not m:
        continue
    module = m.load_module(name)
    for name, value in inspect.getmembers(module):
        if name.startswith('__'):
            continue
        globals()[name] = value
        # print(name)
        __all__.append(name)
    __all__.append(module)
sys.modules[__name__] = Wrapper()



import os
import re
import sys
import shutil
import platform
import subprocess
from pathlib import Path
from typing import Optional

from loguru import logger


def find_binary(name: str, alternative_names: list = None) -> Optional[Path]:
    if alternative_names is None:
        alternative_names = []

    if Path(name).is_file():
        return name

    path = os.environ.get(f"{name.upper()}PATH")
    if path:
        pp = Path(path)
        if not pp.is_file():
            logger.error(f"{p}不是可执行文件！")
            sys.exit(1)
        return pp
    for n in [name] + alternative_names:
        if n in os.listdir(os.getcwd()):
            return Path(os.getcwd()).joinpath(n)
        _which = shutil.which(n)
        if _which:
            return Path(_which)
        if platform.system() == "Windows":
            if f"{n}.exe" in os.listdir(os.getcwd()):
                return Path(os.getcwd()).joinpath(f"{n}.exe")
            _which = shutil.which(f"{n}.exe")
            if _which:
                return Path(_which)

    logger.error(
        f"{name} not found in path, you can specify its binary location "
        f"by setting environment variable: {name.upper()}PATH or put it under the tool folder."
    )
    return None

def build_cmd(binary_name: str, args: str, abort: bool = False) -> str:
    executable = find_binary(binary_name)
    if executable is None:
        if abort:
            sys.exit(1)
        else:
            return ""
    cmd = f'"{executable}" {args}'
    logger.trace(cmd)
    return cmd

def execute_with_output(binary_name: str, args: str, abort: bool = False) -> int:
    cmd = build_cmd(binary_name, args, abort)
    return_code = 0

    def _execute():
        proc = subprocess.Popen(cmd, shell=True, bufsize=1, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                                     universal_newlines=True)
        for stdout in iter(proc.stdout.readline, ""):
            yield stdout
        proc.stdout.close()
        return_code = proc.wait()
    prev = ''
    for out in iter(_execute()):
        out = out.strip()
        if re.sub('(\d| )', '', out) != re.sub('(\d| )', '', prev):
            print(out)
        else:
            print(out, end='\r')
        sys.stdout.flush()
        prev = out
    if return_code != 0:
        logger.warning(f"{binary_name} exit with return code {return_code}")
    return return_code
    

def execute(binary_name: str, args: str, abort: bool = False) -> str:
    cmd = build_cmd(binary_name, args, abort)
    proc = subprocess.run(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    logger.trace(proc)
    ret = "\n".join([proc.stdout.decode(), proc.stderr.decode()])
    if proc.returncode != 0:
        logger.warning(f"{binary_name} exit with return code {proc.returncode}:\n{ret}")
    return ret


def ffmpeg(path: Path, extra_args: str = "") -> str:
    return execute("ffmpeg", f'-i "{path.absolute()}" {extra_args}')


def ffprobe(path: Path) -> str:
    return execute("ffprobe", f'-i "{path.absolute()}"')



import json
from pathlib import Path
from typing import Optional

import requests
from loguru import logger


def ptpimg_upload(img: Path, api_key: str) -> Optional[str]:
    data = {'api_key': api_key}
    files = {'file-upload[0]': open(img, 'rb')}
    req = requests.post('https://ptpimg.me/upload.php', data=data, files=files)

    try:
        res = req.json()
        logger.trace(res)
    except json.decoder.JSONDecodeError:
        res = {}
    if not req.ok:
        logger.trace(req.content)
        logger.warning(
            f"上传图片失败: HTTP {req.status_code}, reason: {req.reason}")
        return None
    if len(res) < 1 or 'code' not in res[0] or 'ext' not in res[0]:
        logger.warning(f"图片直链获取失败")
        return None
    return f"https://ptpimg.me/{res[0].get('code')}.{res[0].get('ext')}"


from typing import List
from pathlib import Path

from torf import Torrent
from loguru import logger

from differential.version import version


def make_torrent_progress(torrent, filepath, pieces_done, pieces_total):
    logger.info(f'制种进度: {pieces_done/pieces_total*100:3.0f} %')


def make_torrent(path: Path, trackers: List[str]):
    logger.info("正在生成种子...")
    t = Torrent(path=path, trackers=trackers,
                comment=f"Generate by Differential {version} made by XGCM")
    t.private = True
    t.generate(callback=make_torrent_progress, interval=1)
    t.write(path.resolve().parent.joinpath(f"{path.name if path.is_dir() else path.stem}.torrent"), overwrite=True)



import re
from pathlib import Path
from decimal import Decimal
from typing import Optional, List

from loguru import logger
from pymediainfo import Track, MediaInfo

from differential.utils.binary import ffprobe


def get_track_attr(
    track: Track, name: str, attr_only: bool = False, use_other: bool = True
) -> Optional[str]:
    alternative_name = None
    if name == "ID":
        alternative_name = "track_id"
    elif name == "Format/Info":
        alternative_name = "format_info"
    elif name == "Codec ID/Info":
        alternative_name = "codec_id_info"
    elif name == "Channel(s)":
        alternative_name = "channel_s"
    elif name == "Bits/(Pixel*Frame)":
        alternative_name = "bits__pixel_frame"

    attr = None
    if alternative_name:
        attr = getattr(track, alternative_name)
    if not attr and use_other:
        attrs = getattr(track, "other_" + name.replace(" ", "_").lower())
        # Always get the first options
        if attrs and len(attrs):
            attr = attrs[0]
    if not attr:
        attr = getattr(track, name.replace(" ", "_").lower())

    if attr:
        return attr if attr_only else "{}: {}".format(name, attr)
    return None


def get_track_attrs(track: Track, names: List[str], join_str: str = " ") -> str:
    attrs = []
    for name in names:
        attr = get_track_attr(track, name, True)
        if attr:
            attrs.append(attr)
    return join_str.join(attrs)


def get_full_mediainfo(mediainfo: MediaInfo) -> str:
    track_format = {
        "general": [
            "Unique ID",
            "Complete name",
            "Format",
            "Format version",
            "Duration",
            "Overall bit rate",
            "Encoded date",
            "Writing application",
            "Writing library",
            "Attachments",
        ],
        "video": [
            "ID",
            "Format",
            "Format/Info",
            "Format profile",
            "Codec ID",
            "Duration",
            "Bit rate",
            "Width",
            "Height",
            "Display aspect ratio",
            "Frame rate mode",
            "Frame rate",
            "Color space",
            "Chroma subsampling",
            "Bit depth",
            "Bits/(Pixel*Frame)",
            "Stream size",
            "Writing library",
            "Encoding settings",
            "Title",
            "Default",
            "Forced",
            "Color range",
            "Color primaries",
            "Transfer characteristics",
            "Matrix coefficients",
            "Mastering display color primaries",
            "Mastering display luminance",
            "Maximum Content Light Level",
            "Maximum Frame-Average Light Level",
        ],
        "audio": [
            "ID",
            "Format",
            "Format/Info",
            "Commercial name",
            "Codec ID",
            "Duration",
            "Bit rate mode",
            "Bit rate",
            "Channel(s)",
            "Channel layout",
            "Sampling rate",
            "Frame rate",
            "Compression mode",
            "Stream size",
            "Title",
            "Language",
            "Service kind",
            "Default",
            "Forced",
        ],
        "text": [
            "ID",
            "Format",
            "Muxing mode",
            "Codec ID",
            "Codec ID/Info",
            "Duration",
            "Bit rate",
            "Count of elements",
            "Stream size",
            "Title",
            "Language",
            "Default",
            "Forced",
        ],
    }
    media_info = ""
    for track_name in track_format.keys():
        for idx, track in enumerate(getattr(mediainfo, "{}_tracks".format(track_name))):
            if len(getattr(mediainfo, "{}_tracks".format(track_name))) > 1:
                media_info += "{} #{}\n".format(track_name.capitalize(), idx + 1)
            else:
                media_info += "{}\n".format(track_name.capitalize())

            media_info += (
                "\n".join(
                    filter(
                        lambda a: a is not None,
                        [
                            get_track_attr(track, name)
                            for name in track_format[track_name]
                        ],
                    )
                )
                + "\n\n"
            )
    # Special treatment with charters
    for track in mediainfo.menu_tracks:
        # Assuming there are always one menu tracks
        media_info += "Menu\n"
        for name in dir(track):
            # TODO: needs improvement
            if name[:2].isdigit():
                media_info += "{} : {}\n".format(
                    name[:-3].replace("_", ":") + "." + name[-3:], getattr(track, name)
                )
        media_info += "\n"
    media_info.strip()
    return media_info

def get_duration(media_info: MediaInfo) -> Optional[Decimal]:
    for track in media_info.tracks:
        if track.track_type == "Video":
            return Decimal(track.duration)
    logger.error(f"未找到视频Track，请检查{main_file}是否为支持的文件")
    return None

def get_resolution(main_file: Path, media_info: MediaInfo) -> Optional[str]:
    # 利用ffprobe获取视频基本信息
    ffprobe_out = ffprobe(main_file)
    m = re.search(r"Stream.*?Video.*?(\d{2,5})x(\d{2,5})", ffprobe_out)
    if not m:
        logger.debug(ffprobe_out)
        logger.warning(f"无法获取到视频的分辨率")
        return None

    # 获取视频分辨率以及长度等信息
    width, height = int(m.group(1)), int(m.group(2))
    for track in media_info.tracks:
        if track.track_type == "Video":
            pixel_aspect_ratio = Decimal(track.pixel_aspect_ratio)
            break
    else:
        logger.error(f"未找到视频Track，请检查{main_file}是否为支持的文件")
        return None

    resolution = None
    if pixel_aspect_ratio <= 1:
        pheight = int(height * pixel_aspect_ratio) + (
            int(height * pixel_aspect_ratio) % 2
        )
        resolution = f"{width}x{pheight}"
    else:
        pwidth = int(width * pixel_aspect_ratio) + (
            int(width * pixel_aspect_ratio) % 2
        )
        resolution = f"{pwidth}x{height}"
    logger.trace(
        f"width: {width} height: {height}, "
        f"PAR: {pixel_aspect_ratio}, resolution: {resolution}"
    )
    return resolution



import argparse

from differential.plugins.nexusphp import NexusPHP


class LemonHD(NexusPHP):

    @classmethod
    def get_help(cls):
        return 'LemonHD插件，适用于LemnonHD电影及电视剧上传'

    @classmethod
    def add_parser(cls, parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
        super().add_parser(parser)
        parser.add_argument('--upload-type', type=str, help="上传类型，默认为电影（movies），其他类型参见柠檬上传URL",
                            default=argparse.SUPPRESS)

    def __init__(self, upload_type='movie', **kwargs):
        super().__init__(upload_url="https://lemonhd.org/upload_{}.php".format(upload_type), **kwargs)



import re
from pathlib import Path
from typing import Optional

import requests
from loguru import logger


def byr_upload(img: Path, authorization: str, url: Optional[str] = None) -> Optional[str]:
    headers = {'authorization': f'{authorization if authorization.startswith("Basic") else "Basic "+authorization}'}
    params = {'command': 'QuickUpload', 'type': 'Images', 'CKEditor': 'descr', 'CKEditorFuncNum': 2}
    files = {'upload': open(img, 'rb')}

    req = requests.post(f"{'https://byr.pt' if not url else url}/ckfinder/core/connector/php/connector.php", params=params, files=files, headers=headers)

    if not req.ok:
        logger.trace(req.content)
        logger.warning(f"上传图片失败: HTTP {req.status_code}, reason: {req.reason}")
        return None
    if url:
        m = re.search(r"\'({}.*?)\'".format(url), req.text)
    else:
        m = re.search(r"\'(https://byr.usx.leishi.io.*?)\'", req.text)
    if not m:
        logger.trace(req.content)
        logger.warning(f"图片直链获取失败")
        return None
    return m.groups()[0]



import json
from pathlib import Path
from typing import Optional

import requests
from loguru import logger


def imgurl_upload(img: Path, url: str, api_key: str) -> Optional[str]:
    data = {'token': api_key}
    files = {'file': open(img, 'rb')}
    req = requests.post(f'{url}/api/upload', data=data, files=files)

    try:
        res = req.json()
        logger.trace(res)
    except json.decoder.JSONDecodeError:
        res = {}
    if not req.ok:
        logger.trace(req.content)
        logger.warning(
            f"上传图片失败: HTTP {req.status_code}, reason: {req.reason} "
            f"{res.get('msg') if 'msg' in res else ''}")
        return None
    if res.get('code') > 200:
        logger.warning(f"上传图片失败: [{res.get('code')}]{res.get('msg')}")
        return None
    return res.get('url')


import argparse

from differential.plugins.base import Base


class NexusPHP(Base):
    @classmethod
    def get_aliases(cls):
        return "nexus", "ne"

    @classmethod
    def get_help(cls):
        return "NexusPHP插件，适用于未经过大规模结构改动的NexusPHP站点"

    @classmethod
    def add_parser(cls, parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
        super().add_parser(parser)
        return parser



from enum import Enum


BOOLEAN_STATES = {'1': True, 'yes': True, 'true': True, 'on': True,
                  '0': False, 'no': False, 'false': False, 'off': False}

BOOLEAN_ARGS = ('generate_nfo', 'make_torrent', 'easy_upload', 'trim_description', 'combine_screenshots', 'use_short_bdinfo', 'use_short_url')

URL_SHORTENER_PATH = 'https://lg.to'


class ImageHosting(Enum):
    PTPIMG = 'ptpimg'
    IMGURL = 'imgurl'
    CHEVERETO = 'chevereto'
    SMMS = 'smms'
    BYR = 'byr'

    @staticmethod
    def parse(s: str):
        if isinstance(s, ImageHosting):
            return s
        if s.lower() == 'ptpimg':
            return ImageHosting.PTPIMG
        elif s.lower() == 'imgurl':
            return ImageHosting.IMGURL
        elif s.lower() == 'chevereto':
            return ImageHosting.CHEVERETO
        elif s.lower() == 'smms':
            return ImageHosting.SMMS
        elif s.lower() == 'byr':
            return ImageHosting.BYR
        raise ValueError(f"不支持的图床：{s}")



from functools import reduce
from differential.torrent import TorrnetBase
from differential.utils.mediainfo import get_full_mediainfo


class AutoFeed(TorrnetBase):

    def __init__(self, plugin: TorrnetBase, seperator: str = '#seperator#'):
        self.plugin = plugin
        self.seperator = seperator

    def __getattribute__(self, name):
        try:
            return super().__getattribute__(name)
        except NotImplementedError:
            if not name.startswith('__') and name in dir(TorrnetBase):
                return getattr(self.plugin, name)
            else:
                raise AttributeError(name)

    @property
    def category(self):
        category = self.plugin.category
        category_map = {
            'concert': '音乐',
            'movie': '电影',
            'documentary': '记录',
            'tvPack': '剧集'
        }
        if category in category_map:
            return category_map[category]
        return category

    @property
    def video_codec(self):
        return self.plugin.video_codec.upper()

    @property
    def audio_codec(self):
        return self.plugin.audio_codec.upper()

    @property
    def resolution(self):
        resolution = self.plugin.resolution
        if resolution == '4320p':
            return '4k'
        elif resolution == '480p':
            return 'SD'
        return resolution

    @property
    def area(self):
        area = self.plugin.area
        area_map = {
            "CN": "大陆",
            "HK": "港台",
            "TW": "港台",
            "US": "欧美",
            "JP": "日本",
            "KR": "韩国",
            "IN": "印度",
            "FR": "欧美",
            "IT": "欧美",
            "GE": "欧美",
            "ES": "欧美",
            "PT": "欧美",
        }
        if area in area_map:
            return area_map[area]
        return area

    @property
    def _raw_info(self) -> dict:
        return {
            # 填充类信息
            'name': self.title, # 主标题
            'small_descr': self.subtitle, # 副标题
            'url': self.imdb_url, # imdb链接
            'dburl': self.douban_url, # 豆瓣链接
            'descr': self.description, # 简介
            'log_info': '',  # 音乐特有
            'tracklist': '', # 音乐特有
            'music_type': '', # 音乐特有
            'music_media': '', # 音乐特有
            'animate_info': '', # 动漫特有|针对北邮人北洋U2的命名方式
            'anidb': '', # 动漫特有
            'torrent_name': '', # 动漫辅助
            'images': self.screenshots, #  截图

            # 选择类信息
            'type': self.category,  # type:可取值——电影/纪录/体育/剧集/动画/综艺……
            'source_sel': self.area, # 来源(地区)：可取值——欧美/大陆/港台/日本/韩国/印度……
            'standard_sel': self.resolution,  # 分辨率：可取值——4K/1080p/1080i/720p/SD
            'audiocodec_sel': self.audio_codec,  # 音频：可取值——AAC/AC3/DTS…………
            'codec_sel': self.video_codec, # 编码：可取值——H264/H265……
            'medium_sel': self.video_type, # 媒介：可取值——web-dl/remux/encode……

            # 其他
            'origin_site': '', # 记录源站点用于跳转后识别
            'origin_url': '', # 记录源站点用于跳转后识别
            'golden_torrent': False, # 主要用于皮转柠檬, 转过去之后会变成字符串
            'mediainfo_cmct': '', # 适用于春天的info
            'imgs_cmct': '', # 适用于春天的截图
            'full_mediainfo': get_full_mediainfo(self.plugin._mediainfo), # 完整的mediainfo有的站点有长短两种，如：铂金家、猫、春天
            'subtitles': [], # 针对皮转海豹，字幕

            'youtube_url': '', # 用于发布iTS
            'ptp_poster': '',  # 用于发布iTS
            'comparisons': '', # 用于海豹
        }

    @property
    def info(self) -> str:
        l = list(reduce(lambda k, v: k + v, self._raw_info.items()))
        return self.seperator + '#linkstr#'.join([str(i) for i in l])



import argparse
from pathlib import Path
from typing import Optional

from loguru import logger

from differential.plugins.chdbits import CHDBits
from differential.version import version

NOTEAM = """
Generate by Differential {} made by
 __   __   _____    _____   __  __ 
 \ \ / /  / ____|  / ____| |  \/  |
  \ V /  | |  __  | |      | \  / |
   > <   | | |_ | | |      | |\/| |
  / . \  | |__| | | |____  | |  | |
 /_/ \_\  \_____|  \_____| |_|  |_|
 
 
""".format(version)

CHD = """
Present by
 $$$$$$\    $$\   $$\   $$$$$$$\  
$$  __$$\   $$ |  $$ |  $$  __$$\ 
$$ /  \__|  $$ |  $$ |  $$ |  $$ |
$$ |        $$$$$$$$ |  $$ |  $$ |
$$ |        $$  __$$ |  $$ |  $$ |
$$ |  $$\   $$ |  $$ |  $$ |  $$ |
\$$$$$$  |  $$ |  $$ |  $$$$$$$  |
 \______/   \__|  \__|  \_______/ 
                              
Generate by Differential {}
 
""".format(version)

CHDPAD = """
Present by
  /$$$$$$  /$$   /$$ /$$$$$$$  /$$$$$$$   /$$$$$$  /$$$$$$$ 
 /$$__  $$| $$  | $$| $$__  $$| $$__  $$ /$$__  $$| $$__  $$
| $$  \__/| $$  | $$| $$  \ $$| $$  \ $$| $$  \ $$| $$  \ $$
| $$      | $$$$$$$$| $$  | $$| $$$$$$$/| $$$$$$$$| $$  | $$
| $$      | $$__  $$| $$  | $$| $$____/ | $$__  $$| $$  | $$
| $$    $$| $$  | $$| $$  | $$| $$      | $$  | $$| $$  | $$
|  $$$$$$/| $$  | $$| $$$$$$$/| $$      | $$  | $$| $$$$$$$/
 \______/ |__/  |__/|_______/ |__/      |__/  |__/|_______/ 
 
Generate by Differential {}
 
""".format(version)


class CHDBitsEncode(CHDBits):

    @classmethod
    def get_help(cls):
        return 'CHDBitsEncode插件，适用于CHDBits压制组'

    @classmethod
    def add_parser(cls, parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
        super().add_parser(parser)
        parser.add_argument('--source-name', type=str, help='压制源名称', default=argparse.SUPPRESS)
        parser.add_argument('--encoder', type=str, help="压制者名字，默认Anonymous", default=argparse.SUPPRESS)
        parser.add_argument('--team', type=str, help="组名，默认CHD", default=argparse.SUPPRESS)

    def __init__(
        self,
        source_name: str,
        encoder: str = "Anonymous",
        team: str = "CHD",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.team = team
        self.encoder = encoder
        self.source_name = source_name

    @property
    def subtitle(self):
        if not self._ptgen.get("site") == "douban":
            return ""
        if 'chinese_title' in self._ptgen:
            return f"{'/'.join([self._ptgen.get('chinese_title')] + self._ptgen.get('aka', []))}"
        else:
            return f"{'/'.join(self._ptgen.get('aka', []))}"

    @property
    def media_info(self):
        media_info = f"{self._main_file.stem}\n"
        imdb_link = self._ptgen.get('imdb_link')
        if imdb_link:
            media_info += f"iMDB URL........: {imdb_link}\n"
        imdb_rating = self._ptgen.get("imdb_rating")
        if imdb_rating:
            media_info += f"iMDB RATiNG.....: {imdb_rating}\n"

        genre = self._imdb.get("genre")
        if genre:
            media_info += f"GENRE...........: {','.join(genre)}\n"
        else:
            genre = self._ptgen.get("genre")
            if genre:
                media_info += f"GENRE...........: {','.join(genre)}\n"

        if self.source_name:
            media_info += f"SOURCE..........: {self.source_name} (Thanks)\n"

        for track in self._mediainfo.general_tracks:
            # TODO(leshi1313): Format it by yourself
            media_info += f"RUNTiME.........: {track.other_duration[0]}\n"
            media_info += f"FilE SiZE.......: {track.other_file_size[0]}\n"
        for track in self._mediainfo.video_tracks:
            media_info += (
                f"ViDEO BiTRATE...: "
                f"{track.encoded_library_name if track.encoded_library_name else track.commercial_name} {track.format_profile} "
                f"@ {track.other_bit_rate[0]}\n"
            )
            media_info += f"FRAME RATE......: {track.other_frame_rate[0]}\n"
            media_info += f"ASPECT RATiO....: {track.other_display_aspect_ratio[0]}\n"
            media_info += f"RESOLUTiON......: {track.width}x{track.height}\n"

        for idx, track in enumerate(self._mediainfo.audio_tracks):
            if track.other_language and len(track.other_language) > 1:
                media_info += (
                    f"AUDiO...........: {'#'+str(idx+1) if len(self._mediainfo.audio_tracks) > 1 else ''} "
                    f"{track.other_language[0]} "
                    f"{track.commercial_name} {track.other_channel_s[0]} "
                    f"@ {track.other_bit_rate[0]}\n"
                )
            else:
                media_info += (
                    f"AUDiO...........: {'#'+str(idx+1) if len(self._mediainfo.audio_tracks) > 1 else ''} "
                    f"{track.commercial_name} {track.other_channel_s[0]} "
                    f"@ {track.other_bit_rate[0]}\n"
                )

        if len(self._mediainfo.text_tracks):
            media_info += "SUBTiTLES.......: {}\n".format(
            " | ".join(
                [
                    f"{track.other_language[0]} {track.format} {track.title if track.title else ''}"
                    if track.other_language and len(track.other_language) > 1
                    else f"{track.format} {track.title if track.title else ''}"
                    for track in self._mediainfo.text_tracks
                ]
            )
        )

        for track in self._mediainfo.menu_tracks:
            if track.chapters_pos_end and track.chapters_pos_begin:
                media_info += f"CHAPTERS........: {int(track.chapters_pos_end) - int(track.chapters_pos_begin)}\n"

        if self.encoder:
            media_info += f"ENCODER.........: {self.encoder} @ {self.team}"
        return media_info

    @property
    def description(self):
        return (
            "[quote][color=Red][size=4][b]"
            "资源及相关素材未经CHD许可 严禁提取发布或二压使用，请注意礼节!"
            "[/b][/color][/quote][/size]\n"
            "{}\n\n"
            "[img]https://www.z4a.net/images/2019/09/13/info.png[/img]"
            "[quote]{}[/quote]\n\n"
            "[img]https://www.z4a.net/images/2019/09/13/screens.png[/img]"
            "\n{}\n"
            "[quote][color=red][b]郑重声明："
            "本站提供的所有影视作品均是在网上搜集任何涉及商业盈利目的均不得使用，"
            "否则产生的一切后果将由您自己承担！本站将不对本站的任何内容负任何法律责任！"
            "该下载内容仅做宽带测试使用，请在下载后24小时内删除。请购买正版！[/b][/color][/quote]".format(
                self._ptgen.get("format"),
                self.media_info + "\n\n" + self.parsed_encoder_log,
                "\n".join([f"[img]{url}[/img]" for url in self._screenshots]),
            )
        )

    @property
    def tags(self):
        tags = super(CHDBitsEncode, self).tags
        tags["official"] = True
        return tags

    def _generate_nfo(self):
        logger.info("正在生成nfo文件...")
        p = Path(self.folder)
        if p.is_file():
            with open(f"{p.parent.joinpath(p.stem)}.nfo", "wb") as f:
                if self.team == "CHD":
                    f.write(CHD.encode())
                elif self.team == "CHDPAD":
                    f.write(CHDPAD.encode())
                else:
                    f.write(NOTEAM.encode())
                f.write(self.media_info.encode())
        elif p.is_dir():
            with open(p.joinpath(f"{p.name}.nfo"), "wb") as f:
                if self.team == "CHD":
                    f.write(CHD.encode())
                elif self.team == "CHDPAD":
                    f.write(CHDPAD.encode())
                else:
                    f.write(NOTEAM.encode())
                f.write(self.media_info.encode())

    @property
    def easy_upload_torrent_info(self):
        torrent_info = super().easy_upload_torrent_info
        torrent_info["team"] = self.team.lower()
        return torrent_info



import string
import argparse
import tempfile
from pathlib import Path
from typing import Optional

from PIL import Image
from loguru import logger

from differential.plugins.lemonhd import LemonHD
from differential.utils.mediainfo import get_track_attr, get_track_attrs


GROUP_QUOTES = {
    'LeagueTV': """
[quote][font=方正粗黑宋简体][size=5][color=Red][b]LeagueTV[/b][/color]高清电视录制小组出品！
[color=blue]本小组接受求片
内地 & 港台电视台 剧集、 电影、 纪录片等
有需求的会员请移步[url=https://lemonhd.org/forums.php?action=viewtopic&forumid=8&topicid=4255]LeagueTV 求片区[/url]跟帖[/size][/font][/quote]

""",
    'LeagueWEB': """
[quote][font=方正粗黑宋简体][size=5][color=Red][b]LeagueWEB[/b][/color]小组出品！
[color=blue]本小组接受求片
内地网络视频平台的 剧集、电影、纪录片等
有需求的会员请移步[url=https://lemonhd.org/forums.php?action=viewtopic&forumid=8&topicid=4557]LeagueWEB应求专用贴[/url]跟帖[/size][/font][/quote]

""",
    'LeagueNF': """
[quote][font=方正粗黑宋简体][size=5][color=Red][b]LeagueNF[/b][/color]小组出品！
[color=blue]本小组接受求片
Netflix流媒体平台 剧集、电影、纪录片、动漫等
有需求的会员请移步[url=https://lemonhd.org/forums.php?action=viewtopic&forumid=8&topicid=4622]LeagueNF小组应求专用贴[/url]跟帖[/size][/font][/quote]

"""
}

class LeagueOfficial(LemonHD):

    @classmethod
    def get_help(cls):
        return 'LeagueOfficial插件，适用于LeagueTV/LeagueWeb/LeagueNF官组电影及电视剧上传'

    @classmethod
    def add_parser(cls, parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
        super().add_parser(parser)
        parser.add_argument('--source-name', type=str, help='资源来源，HDTV/WEB-DL等', default=argparse.SUPPRESS)
        parser.add_argument('--uploader', type=str, help="发布者者名字，默认Anonymous", default=argparse.SUPPRESS)
        parser.add_argument('--team', type=str, help="官组名，LeagueTV/LeagueWeb/LeagueNF等等", default=argparse.SUPPRESS)
        parser.add_argument('--combine-screenshots', type=bool, help='是否合并所有截图为一张图，默认开启', default=argparse.SUPPRESS)
        return parser

    def __init__(self, source_name: str, team: str, uploader: str = "Anonymous", combine_screenshots: bool = True, **kwargs):
        super().__init__(**kwargs)
        self.team = team
        self.uploader = uploader
        self.source_name = source_name
        self.combine_screenshots = combine_screenshots

    def _make_screenshots(self) -> Optional[str]:
        screenshots_dir = super()._make_screenshots()
        if not self.combine_screenshots:
            return screenshots_dir

        logger.info("正在合并图片...")
        images = [Image.open(i) for i in sorted(Path(screenshots_dir).glob("*.png"))]

        width, height = images[0].size
        new_width = 2 * width
        new_height = (self.screenshot_count // 2 + self.screenshot_count % 2) * height

        new_im = Image.new('RGBA', (new_width, new_height))
        for idx, im in enumerate(images):
            x = (idx % 2) * width
            y = (idx // 2) * height
            new_im.paste(im, (x, y))

        temp_dir = tempfile.mkdtemp()
        screenshot_path = f'{temp_dir}/{self._main_file.stem}.thumb.png'
        new_im.save(screenshot_path, format='png')
        return temp_dir

    @property
    def subtitle(self):
        if not self._ptgen.get("site") == "douban":
            return ""
        subtitle = f"{'/'.join(self._ptgen.get('this_title', []) + self._ptgen.get('aka', []))} "
        if self._ptgen.get('cast'):
            subtitle += f"[主演: {'/'.join([c.get('name').strip(string.ascii_letters+string.whitespace) for c in self._ptgen.get('cast')[:3]])}]"
        return subtitle

    @property
    def media_info(self):
        media_info = ""
        for track in self._mediainfo.general_tracks:
            media_info += f"File Name............: {track.file_name}\n"
            media_info += f"File Size............: {get_track_attr(track, 'file_size', True)}\n"
            media_info += f"Duration.............: {get_track_attr(track, 'duration', True)}\n"
            media_info += f"Bit Rate.............: {get_track_attr(track, 'overall_bit_rate', True)}\n"
        for track in self._mediainfo.video_tracks:
            media_info += f"Video Codec..........: {get_track_attr(track, 'format', True)} {get_track_attr(track, 'format profile', True)}\n"
            media_info += f"Frame Rate...........: {get_track_attr(track, 'frame rate', True)}\n"
            media_info += f"Resolution...........: {get_track_attr(track, 'width', True, False)} x {get_track_attr(track, 'height', True, False)}\n"
            media_info += f"Display Ratio........: {get_track_attr(track, 'display_aspect_ratio', True)}\n"
            media_info += f"Scan Type............: {get_track_attr(track, 'scan type', True)}\n"
            media_info += f"Bite Depth...........: {get_track_attr(track, 'bit depth', True)}\n"

        for idx, track in enumerate(self._mediainfo.audio_tracks):
            media_info += f"Audio #{idx}.............: {get_track_attrs(track, ['bit rate', 'bit rate mode','channel_s', 'format'])}\n"

        for idx, track in enumerate(self._mediainfo.text_tracks):
            media_info += f"Subtitle #{idx}..........: {get_track_attrs(track, ['format', 'title', 'language'])}\n"

        if self.source_name:
            media_info += f"Source...............: {self.source_name}\n"
        media_info += f"Uploader.............: {self.uploader} @ {self.team}"
        return media_info

    @property
    def description(self):
        return (
            "{}"
            "{}\n\n"
            "[img]https://imgbox.leaguehd.com/images/2021/01/04/info_01.png[/img]\n"
            "[quote][size=3][color=Navy][b]★★★ ★★ General Information ★★★ ★★ [/color][/size][/b]\n"
            "[font=Courier New]{}[/font][/quote]\n\n"
            "[img]https://imgbox.leaguehd.com/images/2021/01/04/screens_01.png[/img]\n"
            "{}\n".format(
                GROUP_QUOTES.get(self.team, ''),
                self._ptgen.get("format"),
                self.media_info,
                "\n".join([f"[img]{url}[/img]" for url in self._screenshots]),
            )
        )



import re
import json
from pathlib import Path
from typing import Optional

import requests
from loguru import logger

sessions = {}


def chevereto_api_upload(img: Path, url: str, api_key: str) -> Optional[str]:
    data = {'key': api_key}
    files = {'source': open(img, 'rb')}
    req = requests.post(f'{url}/api/1/upload', data=data, files=files)

    try:
        res = req.json()
        logger.trace(res)
    except json.decoder.JSONDecodeError:
        res = {}
    if not req.ok:
        logger.trace(req.content)
        logger.warning(
            f"上传图片失败: HTTP {req.status_code}, reason: {req.reason} "
            f"{res['error'].get('message') if 'error' in res else ''}")
        return None
    if 'error' in res:
        logger.warning(f"上传图片失败: [{res['error'].get('code')}]{res['error'].get('message')}")
        return None
    if 'image' not in res or 'url' not in res['image']:
        logger.warning(f"图片直链获取失败")
        return None
    return res['image']['url']


def chevereto_cookie_upload(img: Path, url: str, cookie: str, auth_token: str) -> Optional[str]:
    headers = {'cookie': cookie}
    data = {'type': 'file', 'action': 'upload', 'nsfw': 0, 'auth_token': auth_token}
    files = {'source': open(img, 'rb')}
    req = requests.post(f'{url}/json', data=data, files=files, headers=headers)

    try:
        res = req.json()
        logger.trace(res)
    except json.decoder.JSONDecodeError:
        res = {}
    if not req.ok:
        logger.trace(req.content)
        logger.warning(
            f"上传图片失败: HTTP {req.status_code}, reason: {req.reason} "
            f"{res['error'].get('message') if 'error' in res else ''}")
        return None
    if 'error' in res:
        logger.warning(f"上传图片失败: [{res['error'].get('code')}] {res['error'].get('context')} {res['error'].get('message')}")
        return None
    if 'status_code' in res and res.get('status_code') != 200:
        logger.warning(f"上传图片失败: [{res['status_code']}] {res.get('status_txt')}")
        return None 
    if 'image' not in res or 'url' not in res['image']:
        logger.warning(f"图片直链获取失败")
        return None
    return res['image']['url']


def with_session(func):
    def wrapper(img: Path, url: str, username: str, password: str):
        if (username, password) not in sessions:
            session = requests.Session()
            req = session.get(url)
            m = re.search(r'auth_token.*?\"(\w+)\"', req.text)
            if not m:
                logger.warning("未找到auth_token，请重试")
                return
            auth_token = m.groups()[0]
            data = {'auth_token': auth_token, 'login-subject': username, 'password': password, 'keep-login': 1}
            logger.info("正在登录Chevereto...")
            req = session.post(f"{url}/login", data=data)
            if not req.ok:
                logger.warning("Chevereto登录失败，请重试")
                return
            sessions[(username, password)] = (session, auth_token)
        else:
            session, auth_token = sessions.get((username, password))
        return func(session, img, url, auth_token)
    return wrapper


@with_session
def chevereto_username_upload(session: requests.Session, img: Path, url: str, auth_token: str) -> Optional[str]:
    data = {'type': 'file', 'action': 'upload', 'nsfw': 0, 'auth_token': auth_token}
    files = {'source': open(img, 'rb')}
    req = session.post(f'{url}/json', data=data, files=files)

    try:
        res = req.json()
        logger.trace(res)
    except json.decoder.JSONDecodeError:
        res = {}
    if not req.ok:
        logger.trace(req.content)
        logger.warning(
            f"上传图片失败: HTTP {req.status_code}, reason: {req.reason} "
            f"{res['error'].get('message') if 'error' in res else ''}")
        return None
    if 'error' in res:
        logger.warning(f"上传图片失败: [{res['error'].get('code')}] {res['error'].get('context')} {res['error'].get('message')}")
        return None
    if 'status_code' in res and res.get('status_code') != 200:
        logger.warning(f"上传图片失败: [{res['status_code']}] {res.get('status_txt')}")
        return None 
    if 'image' not in res or 'url' not in res['image']:
        logger.warning(f"图片直链获取失败")
        return None
    return res['image']['url']


#!/usr/bin/env python
from glob import glob
from setuptools import setup, find_packages

setup(
    name="Differential",
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'differential': glob('tools/**/*')
    },
    long_description=open("README.md", "r", encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    description="a Python script for easy uploading torrents to varies PT sites.",
    author="Lei Shi",
    author_email="me@leishi.io",
    url="https://github.com/leishi1313/Differential",
    keywords=["PT", "mediainfo", "ptgen", "ptpimg"],
    classifiers=[
        "Environment :: Console",
        "Operating System :: MacOS",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3.6.2",
    install_requires=[
        "loguru>=0.5.0",
        "Pillow>=8.0.0",
        "pymediainfo>=5.0",
        "torf>=3.0.0",
        "requests",
    ],
    entry_points={
        "console_scripts": [
            "differential=differential.main:main",
            "dft=differential.main:main",
        ]
    },
)



import argparse
from configparser import RawConfigParser

from differential.constants import ImageHosting, BOOLEAN_ARGS, BOOLEAN_STATES


def merge_config(args: argparse.Namespace) -> dict:
    merged = {}
    config = None
    if hasattr(args, 'config'):
        config = RawConfigParser()
        config.read(args.config, encoding='utf-8')

    if config:
        # First use the args in the general section
        for arg in config.defaults().keys():
            merged[arg] = config.defaults()[arg]

        # Then use the args from config file matching the plugin name
        if args.plugin in config.sections():
            for arg in config[args.plugin].keys():
                merged[arg] = config[args.plugin][arg]

    # Args from command line has the highest priority
    for arg in vars(args):
        merged[arg] = getattr(args, arg)

    # Handling non-str non-int args
    if 'image_hosting' in merged:
        merged['image_hosting'] = ImageHosting.parse(merged['image_hosting'])
    if any(arg in BOOLEAN_ARGS for arg in merged.keys()):
        for arg in BOOLEAN_ARGS:
            if arg in merged and not isinstance(merged[arg], bool):
                # Might be buggy to always assume not recognized args is False
                merged[arg] = BOOLEAN_STATES.get(merged[arg], False)

    # Parse int args
    for arg in merged:
        if isinstance(merged[arg], str) and merged[arg].isdigit():
            merged[arg] = int(merged[arg])
    return merged




from differential.torrent import TorrnetBase


class EasyUpload(TorrnetBase):

    def __init__(self, plugin: TorrnetBase):
        self.plugin = plugin

    def __getattribute__(self, name):
        try:
            return super().__getattribute__(name)
        except NotImplementedError:
            if not name.startswith('__') and name in dir(TorrnetBase):
                return getattr(self.plugin, name)
            else:
                raise AttributeError(name)

    @property
    def area(self):
        # map EU coutries to EU
        area = self.plugin.area
        area_map = {
            "FR": "EU",
            "IT": "EU",
            "GE": "EU",
            "ES": "EU",
            "PT": "EU",
        }
        if area in area_map:
            return area_map[area]
        return area

    @property
    def torrent_info(self) -> dict:
        return {
            "title": self.title,
            "subtitle": self.subtitle,
            "description": self.description,
            "originalDescription": self.original_description,
            "doubanUrl": self.douban_url,
            "doubanInfo": self.douban_info,
            "imdbUrl": self.imdb_url,
            "mediaInfo": self.media_info,
            "mediaInfos": self.media_infos,
            "screenshots": self.screenshots,
            "poster": self.poster,
            "year": self.year,
            "category": self.category,
            "videoType": self.video_type,
            "format": self.format,
            "source": self.source,
            "videoCodec": self.video_codec,
            "audioCodec": self.audio_codec,
            "resolution": self.resolution,
            "area": self.area,
            "movieAkaName": self.movie_aka_name,
            "movieName": self.movie_name,
            "size": self.size,
            "tags": self.tags,
            "otherTags": self.other_tags,
            "comparisons": self.comparisons,
            "isForbidden": False,
            "sourceSiteType": "NexusPHP",
        }



from differential.utils.image.byr import byr_upload
from differential.utils.image.smms import smms_upload
from differential.utils.image.ptpimg import ptpimg_upload
from differential.utils.image.imgurl import imgurl_upload
from differential.utils.image.chevereto import chevereto_api_upload, chevereto_cookie_upload, chevereto_username_upload


import os
import re
import sys
import json
import shutil
import platform
import tempfile
import argparse
from pathlib import Path
from typing import Optional
from urllib.parse import quote
from abc import ABC, ABCMeta, abstractmethod

import requests
from PIL import Image
from loguru import logger
from pymediainfo import MediaInfo

from differential import tools
from differential.torrent import TorrnetBase
from differential.version import version
from differential.constants import ImageHosting
from differential.utils.browser import open_link
from differential.utils.torrent import make_torrent
from differential.utils.parse import parse_encoder_log
from differential.utils.uploader import EasyUpload, AutoFeed
from differential.utils.binary import ffprobe, execute, execute_with_output
from differential.utils.mediainfo import (
    get_full_mediainfo,
    get_resolution,
    get_duration,
)
from differential.utils.image import (
    byr_upload,
    ptpimg_upload,
    smms_upload,
    imgurl_upload,
    chevereto_api_upload,
    chevereto_username_upload,
)


PARSER = argparse.ArgumentParser(description="Differential - 差速器 PT快速上传工具")
PARSER.add_argument(
    "-v",
    "--version",
    help="显示差速器当前版本",
    action="version",
    version=f"Differential {version}",
)
subparsers = PARSER.add_subparsers(help="使用下列插件名字来查看插件的详细用法")
REGISTERED_PLUGINS = {}


class PluginRegister(ABCMeta):
    def __init__(cls, name, bases, attrs):
        super().__init__(name, bases, attrs)
        # Skip base class
        if name != "Base" and name not in REGISTERED_PLUGINS:
            aliases = (name.lower(),)
            if "get_aliases" in cls.__dict__:
                aliases += cls.get_aliases()
            subparser = subparsers.add_parser(
                name, aliases=aliases, help=cls.get_help()
            )
            subparser.set_defaults(plugin=name)
            cls.add_parser(subparser)
            for n in aliases:
                REGISTERED_PLUGINS[n] = cls
            REGISTERED_PLUGINS[name] = cls

    @classmethod
    @abstractmethod
    def get_help(mcs):
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def get_aliases(mcs):
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def add_parser(mcs, parser: argparse.ArgumentParser):
        raise NotImplementedError()


class Base(ABC, TorrnetBase, metaclass=PluginRegister):
    @classmethod
    @abstractmethod
    def add_parser(cls, parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
        parser.add_argument(
            "-c",
            "--config",
            type=str,
            help="配置文件的路径，默认为config.ini",
            default="config.ini",
        )
        parser.add_argument(
            "-l", "--log", type=str, help="log文件的路径", default=argparse.SUPPRESS
        )
        parser.add_argument(
            "-f", "--folder", type=str, help="种子文件夹的路径", default=argparse.SUPPRESS
        )
        parser.add_argument(
            "-u", "--url", type=str, help="豆瓣链接", default=argparse.SUPPRESS
        )
        parser.add_argument(
            "-uu",
            "--upload-url",
            type=str,
            help="PT站点上传的路径，一般为https://xxxxx.com/upload.php",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "-t",
            "--make-torrent",
            action="store_true",
            help="是否制种，默认否",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "-n",
            "--generate-nfo",
            action="store_true",
            help="是否用MediaInfo生成NFO文件，默认否",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "-s",
            "--screenshot-count",
            type=int,
            help="截图数量，默认为0，即不生成截图",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--use-short-bdinfo",
            action="store_true",
            help="使用QUICK SUMMARY作为BDInfo，默认使用完整BDInfo",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--optimize-screenshot",
            action="store_true",
            help="是否压缩截图（无损），默认压缩",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--image-hosting",
            type=ImageHosting,
            help=f"图床的类型，现在支持{','.join(i.value for i in ImageHosting)}",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--ptpimg-api-key",
            type=str,
            help="PTPIMG的API Key",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--chevereto-hosting-url",
            type=str,
            help="自建chevereto图床的地址",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--chevereto-api-key",
            type=str,
            help="自建Chevereto的API Key，详情见https://v3-docs.chevereto.com/api/#api-call",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--chevereto-username",
            type=str,
            help="如果自建Chevereto的API未开放，请设置username和password",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--chevereto-password",
            type=str,
            help="如果自建Chevereto的API未开放，请设置username和password",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--imgurl-api-key",
            type=str,
            help="Imgurl的API Key",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--smms-api-key", type=str, help="SM.MS的API Key", default=argparse.SUPPRESS
        )
        parser.add_argument(
            "--byr-authorization",
            type=str,
            help="BYR的Authorization头，可登录后访问任意页面F12查看，形如Basic xxxxxxxxxxxxxxxx==",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--byr-alternative-url",
            type=str,
            help="BYR反代地址(如有)，可为空",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--ptgen-url", type=str, help="自定义PTGEN的地址", default=argparse.SUPPRESS
        )
        parser.add_argument(
            "--ptgen-retry", type=int, help="PTGEN重试次数，默认为3次", default=argparse.SUPPRESS
        )
        parser.add_argument(
            "--announce-url", type=str, help="制种时announce地址", default=argparse.SUPPRESS
        )

        parser.add_argument(
            "--encoder-log", type=str, help="压制log的路径", default=argparse.SUPPRESS
        )
        upload_option_group = parser.add_mutually_exclusive_group()
        upload_option_group.add_argument(
            "--easy-upload",
            action="store_true",
            help="使用树大Easy Upload插件自动填充",
            dest="easy_upload",
            default=argparse.SUPPRESS,
        )
        upload_option_group.add_argument(
            "--auto-feed",
            action="store_true",
            help="使用明日大Auto Feed插件自动填充",
            dest="auto_feed",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--trim-description",
            action="store_true",
            help="是否在生成的链接中省略种子描述，该选项主要是为了解决浏览器限制URL长度的问题，默认关闭",
            default=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--use-short-url",
            action="store_true",
            help="是否缩短生成的上传链接",
            default=argparse.SUPPRESS,
        )
        return parser

    def __init__(
        self,
        folder: str,
        url: str,
        upload_url: str,
        screenshot_count: int = 0,
        optimize_screenshot: bool = True,
        use_short_bdinfo: bool = False,
        image_hosting: ImageHosting = ImageHosting.PTPIMG,
        chevereto_hosting_url: str = "",
        imgurl_hosting_url: str = "",
        ptpimg_api_key: str = None,
        chevereto_api_key: str = None,
        chevereto_username: str = None,
        chevereto_password: str = None,
        imgurl_api_key: str = None,
        smms_api_key: str = None,
        byr_authorization: str = None,
        byr_alternative_url: str = None,
        ptgen_url: str = "https://ptgen.lgto.workers.dev",
        announce_url: str = "https://example.com",
        ptgen_retry: int = 3,
        generate_nfo: bool = False,
        make_torrent: bool = False,
        easy_upload: bool = False,
        auto_feed: bool = False,
        trim_description: bool = False,
        use_short_url: bool = False,
        encoder_log: str = "",
        **kwargs,
    ):
        self.folder = Path(folder)
        self.url = url
        self.upload_url = upload_url
        self.screenshot_count = screenshot_count
        self.optimize_screenshot = optimize_screenshot
        self.use_short_bdinfo = use_short_bdinfo
        self.image_hosting = image_hosting
        self.chevereto_hosting_url = chevereto_hosting_url
        self.imgurl_hosting_url = imgurl_hosting_url
        self.ptpimg_api_key = ptpimg_api_key
        self.chevereto_username = chevereto_username
        self.chevereto_password = chevereto_password
        self.chevereto_api_key = chevereto_api_key
        self.imgurl_api_key = imgurl_api_key
        self.smms_api_key = smms_api_key
        self.byr_authorization = byr_authorization
        self.byr_alternative_url = byr_alternative_url
        self.ptgen_url = ptgen_url
        self.announce_url = announce_url
        self.ptgen_retry = ptgen_retry
        self.generate_nfo = generate_nfo
        self.make_torrent = make_torrent
        self.easy_upload = easy_upload
        self.auto_feed = auto_feed
        self.trim_description = trim_description
        self.use_short_url = use_short_url
        self.encoder_log = encoder_log

        self.is_bdmv = False
        self._bdinfo = None
        self._main_file: Optional[Path] = None
        self._ptgen: dict = {}
        self._imdb: dict = {}
        self._mediainfo: Optional[MediaInfo] = None
        self._screenshots: list = []

    def upload_screenshots(self, img_dir: str) -> list:
        img_urls = []
        for count, img in enumerate(sorted(Path(img_dir).glob("*.png"))):
            if img.is_file():
                img_url = None
                img_url_file = img.resolve().parent.joinpath(
                    ".{}.{}".format(self.image_hosting.value, img.stem)
                )
                if img_url_file.is_file():
                    with open(img_url_file, "r") as f:
                        img_url = f.read().strip()
                        logger.info(f"发现已上传的第{count + 1}张截图链接：{img_url}")
                else:
                    if self.image_hosting == ImageHosting.PTPIMG:
                        img_url = ptpimg_upload(img, self.ptpimg_api_key)
                    elif self.image_hosting == ImageHosting.CHEVERETO:
                        if not self.chevereto_hosting_url:
                            logger.error("Chevereto地址未提供，请设置chevereto_hosting_url")
                            sys.exit(1)
                        if self.chevereto_api_key:
                            img_url = chevereto_api_upload(
                                img, self.chevereto_hosting_url, self.chevereto_api_key
                            )
                        elif self.chevereto_username and self.chevereto_password:
                            img_url = chevereto_username_upload(
                                img,
                                self.chevereto_hosting_url,
                                self.chevereto_username,
                                self.chevereto_password,
                            )
                        else:
                            logger.error(
                                "Chevereto的API或用户名或密码未设置，请检查chevereto-username/chevereto-password设置"
                            )
                    elif self.image_hosting == ImageHosting.IMGURL:
                        img_url = imgurl_upload(
                            img, self.imgurl_hosting_url, self.imgurl_api_key
                        )
                    elif self.image_hosting == ImageHosting.SMMS:
                        img_url = smms_upload(img, self.smms_api_key)
                    elif self.image_hosting == ImageHosting.BYR:
                        img_url = byr_upload(
                            img, self.byr_authorization, self.byr_alternative_url
                        )

                if img_url:
                    logger.info(f"第{count + 1}张截图地址：{img_url}")
                    with open(img_url_file, "w") as f:
                        f.write(img_url)
                    img_urls.append(img_url)
                else:
                    logger.info(f"第{count + 1}张截图上传失败，请自行上传：{img.resolve()}")
        return img_urls

    def _get_ptgen(self) -> dict:
        self._imdb = {}
        ptgen_failed = {"format": "PTGen获取失败，请自行获取相关信息", "failed": True}
        logger.info(f"正在获取PTGen: {self.url}")
        params = {"url": self.url}
        req = requests.get(self.ptgen_url, params)
        if not req.ok:
            logger.trace(req.content)
            logger.warning(f"获取PTGen失败: HTTP f{req.status_code}, reason: {req.reason}")
            return ptgen_failed
        if not req.json().get("success", False):
            logger.trace(req.json())
            logger.warning(f"获取PTGen失败: {req.json().get('error', 'Unknown error')}")
            return ptgen_failed

        # 尝试获取IMDB描述
        if req.json().get("site") != "imdb":
            if req.json().get("imdb_link"):
                imdb_params = {"url": req.json().get("imdb_link")}
                imdb_req = requests.get(self.ptgen_url, imdb_params)
                if imdb_req.ok and imdb_req.json().get("success"):
                    self._imdb = imdb_req.json()
        else:
            self._imdb = req.json()
        logger.info(f"获取PTGen成功 {req.json().get('chinese_title', '')}")
        return req.json()

    def _get_bdinfo(self) -> str:
        logger.info("目标为BDMV，正在扫描BDInfo...")
        temp_dir = tempfile.mkdtemp()
        bdinfos = []
        for f in self.folder.glob("**/BDMV"):
            logger.info(f"正在扫描{f.parent}...")
            if platform.system() == "Windows":
                execute_with_output(
                    os.path.join(
                        os.path.dirname(tools.__file__), "BDinfoCli.0.7.3\BDInfo.exe"
                    ),
                    f'-w "{f.parent}" {temp_dir}',
                    abort=True,
                )
            else:
                execute_with_output(
                    "mono",
                    f""""{os.path.join(os.path.dirname(tools.__file__), 'BDinfoCli.0.7.3/BDInfo.exe')}" -w """
                    f'"{f.parent}" {temp_dir}',
                    abort=True,
                )
        for info in Path(temp_dir).glob("*.txt"):
            with info.open("r") as f:
                content = f.read()
            if self.use_short_bdinfo:
                m = re.search(r"(QUICK SUMMARY:\n+(.+?\n)+)\n\n", content)
                if m:
                    bdinfos.append(m.groups()[0])
            else:
                m = re.search(
                    r"(DISC INFO:\n+(.+?\n{1,2})+?)(?:CHAPTERS:\n|STREAM DIAGNOSTICS:\n|\[\/code\]\n<---- END FORUMS PASTE ---->)",
                    content,
                )
                if m:
                    bdinfos.append(m.groups()[0])
        shutil.rmtree(temp_dir, ignore_errors=True)
        return "\n\n".join(bdinfos)

    def _find_mediainfo(self) -> MediaInfo:
        # Always find the biggest file in the folder
        logger.info(f"正在获取Mediainfo: {self.folder}")
        if self.folder.is_file():
            self._main_file = self.folder
        else:
            logger.info("目标为文件夹，正在获取最大的文件...")
            biggest_size = -1
            biggest_file = None
            has_bdmv = False
            for f in self.folder.glob("**/*"):
                if f.is_file():
                    if f.suffix == ".bdmv":
                        has_bdmv = True
                    s = os.stat(f.absolute()).st_size
                    if s > biggest_size:
                        biggest_size = s
                        biggest_file = f
            if biggest_file:
                self._main_file = biggest_file
        if self._main_file is None:
            logger.error("未在目标目录找到文件！")
            sys.exit(1)
        elif self._main_file.suffix == '.iso':
            logger.error("请将iso文件挂载后再使用差速器")
            sys.exit(1)
        mediainfo = MediaInfo.parse(self._main_file)
        logger.info(f"已获取Mediainfo: {self._main_file}")
        logger.trace(mediainfo.to_data())
        if has_bdmv:
            self.is_bdmv = True
            self._bdinfo = self._get_bdinfo()
        return mediainfo

    def _generate_nfo(self):
        logger.info("正在生成nfo文件...")
        if self.folder.is_file():
            with open(
                f"{self.folder.resolve().parent.joinpath(self.folder.stem)}.nfo", "wb"
            ) as f:
                f.write(self.media_info.encode())
        elif self.folder.is_dir():
            with open(self.folder.joinpath(f"{self.folder.name}.nfo"), "wb") as f:
                f.write(self.media_info.encode())

    def _make_screenshots(self) -> Optional[str]:
        resolution = get_resolution(self._main_file, self._mediainfo)
        duration = get_duration(self._mediainfo)
        if resolution is None or duration is None:
            return None

        temp_dir = None
        # 查找已有的截图
        for f in Path(tempfile.gettempdir()).glob("Differential*"):
            if f.is_dir() and self.folder.name in f.name:
                if (
                    self.screenshot_count > 0
                    and len(list(f.glob("*.png"))) == self.screenshot_count
                ):
                    temp_dir = f.absolute()
                    logger.info("发现已生成的{}张截图，跳过截图...".format(self.screenshot_count))
                    break
        else:
            temp_dir = tempfile.mkdtemp(
                prefix="Differential.{}.".format(version), suffix=self.folder.name
            )
            # 生成截图
            for i in range(1, self.screenshot_count + 1):
                logger.info(f"正在生成第{i}张截图...")
                t = int(i * duration / (self.screenshot_count + 1))
                screenshot_path = (
                    f"{temp_dir}/{self._main_file.stem}.thumb_{str(i).zfill(2)}.png"
                )
                execute(
                    "ffmpeg",
                    (
                        f'-y -ss {t}ms -skip_frame nokey -i "{self._main_file.absolute()}" '
                        f'-s {resolution} -vsync 0 -vframes 1 -c:v png "{screenshot_path}"'
                    ),
                )
                if self.optimize_screenshot:
                    image = Image.open(screenshot_path)
                    image.save(f"{screenshot_path}", format="PNG", optimized=True)
        return temp_dir

    def _get_screenshots(self) -> list:
        if self._main_file is None:
            logger.error("未找到可以被截图的资源，请确认目标目录含有支持的资源!")
            return []

        temp_dir = self._make_screenshots()
        if temp_dir is None:
            return []

        # 上传截图
        screenshots = self.upload_screenshots(temp_dir)
        logger.trace(f"Collected screenshots: {screenshots}")

        # 删除临时文件夹
        # shutil.rmtree(temp_dir, ignore_errors=True)

        return screenshots

    def _prepare(self):
        ptgen_retry = self.ptgen_retry
        self._ptgen = self._get_ptgen()
        while self._ptgen.get("failed") and ptgen_retry > 0:
            self._ptgen = self._get_ptgen()
            ptgen_retry -= 1
        self._mediainfo = self._find_mediainfo()
        if self.generate_nfo:
            self._generate_nfo()
        self._screenshots = self._get_screenshots()
        if self.make_torrent:
            make_torrent(self.folder, [self.announce_url])

    @property
    def parsed_encoder_log(self):
        return parse_encoder_log(self.encoder_log)

    @property
    def title(self):
        # TODO: Either use file name or generate from mediainfo and ptgen
        temp_name = (
            self.folder.name if self.folder.is_dir() else self.folder.stem
        ).replace(".", " ")
        temp_name = re.sub(r'(?<=5|7)( )1(?=.*$)', '.1', temp_name)
        return temp_name

    @property
    def subtitle(self):
        if not self._ptgen.get("site") == "douban":
            return ""
        if "chinese_title" in self._ptgen:
            subtitle = f"{'/'.join([self._ptgen.get('chinese_title')] + self._ptgen.get('aka', []))}"
        else:
            subtitle = f"{'/'.join(self._ptgen.get('aka', []))}"
        if self._ptgen.get("director"):
            subtitle += (
                f"【导演：{'/'.join([d.get('name') for d in self._ptgen.get('director')])}】"
            )
        if self._ptgen.get("writer"):
            subtitle += (
                f"【编剧：{'/'.join([w.get('name') for w in self._ptgen.get('writer')])}】"
            )
        if self._ptgen.get("cast"):
            subtitle += (
                f"【主演：{'/'.join([c.get('name') for c in self._ptgen.get('cast')[:3]])}】"
            )
        return subtitle

    @property
    def media_info(self):
        if self.is_bdmv:
            return self._bdinfo
        else:
            return get_full_mediainfo(self._mediainfo)

    @property
    def media_infos(self):
        return []

    @property
    def description(self):
        return "{}\n\n[quote]{}{}[/quote]\n\n{}".format(
            self._ptgen.get("format"),
            self.media_info,
            "\n\n" + self.parsed_encoder_log if self.parsed_encoder_log else "",
            "\n".join([f"[img]{url}[/img]" for url in self._screenshots]),
        )

    @property
    def original_description(self):
        return ""

    @property
    def douban_url(self):
        if self._ptgen.get("site") == "douban":
            return f"https://movie.douban.com/subject/{self._ptgen.get('sid')}"
        return ""

    @property
    def douban_info(self):
        return ""

    @property
    def imdb_url(self):
        return self._ptgen.get("imdb_link", "")

    @property
    def screenshots(self):
        return self._screenshots

    @property
    def poster(self):
        return self._ptgen.get("poster", "")

    @property
    def year(self):
        return self._ptgen.get("year", "")

    @property
    def category(self):
        if "演唱会" in self._ptgen.get("tags", []) and "音乐" in self._ptgen.get(
            "genre", []
        ):
            return "concert"
        imdb_genre = self._imdb.get("genre", [])
        if "Documentary" in imdb_genre:
            return "documentary"
        imdb_type = self._imdb.get("@type", "")
        if imdb_type == "Movie":
            return "movie"
        if imdb_type == "TVSeries":
            return "tvPack"
        return imdb_type

    @property
    def video_type(self):
        if "webdl" in self.folder.name.lower() or "web-dl" in self.folder.name.lower():
            return "web"
        elif "remux" in self.folder.name.lower():
            return "remux"
        elif "hdtv" in self.folder.name.lower():
            return "hdtv"
        elif any(e in self.folder.name.lower() for e in ("x264", "x265")):
            return "encode"
        elif "bluray" in self.folder.name.lower() and not any(
            e in self.folder.name.lower() for e in ("x264", "x265")
        ):
            return "bluray"
        elif "uhd" in self.folder.name.lower():
            return "uhdbluray"
        for track in self._mediainfo.tracks:
            if track.track_type == "Video":
                if track.encoding_settings:
                    return "encode"
        return ""

    @property
    def format(self):
        # TODO: Maybe read from mediainfo
        return self._main_file.suffix

    @property
    def source(self):
        return ""

    @property
    def video_codec(self):
        for track in self._mediainfo.tracks:
            if track.track_type == "Video":
                if track.encoded_library_name:
                    return track.encoded_library_name
                if track.commercial_name == "AVC":
                    return "h264"
                if track.commercial_name == "HEVC":
                    return "hevc"
        #  h264: "AVC/H.264",
        #  hevc: "HEVC",
        #  x264: "x264",
        #  x265: "x265",
        #  h265: "HEVC",
        #  mpeg2: "MPEG-2",
        #  mpeg4: "AVC/H.264",
        #  vc1: "VC-1",
        #  dvd: "MPEG"
        return ""

    @property
    def audio_codec(self):
        codec_map = {
            "AAC": "aac",
            "Dolby Digital Plus": "dd+",
            "Dolby Digital": "dd",
            "DTS-HD Master Audio": "dtshdma",
            "Dolby Digital Plus with Dolby Atmos": "atmos",
            "Dolby TrueHD": "truehd",
            "Dolby TrueHD with Dolby Atmos": "truehd",
        }
        for track in self._mediainfo.tracks:
            if track.track_type == "Audio":
                if track.format_info == "Audio Coding 3":
                    return "ac3"
                if track.format_info == "Free Lossless Audio Codec":
                    return "flac"
                if track.commercial_name in codec_map:
                    return codec_map.get(track.commercial_name)
                # TODO: other formats
                # dts: "3",
                # lpcm: "21",
                # dtsx: "3",
                # ape: "2",
                # wav: "22",
                # mp3: "4",
                # m4a: "5",
                # other: "7"
        return ""

    @property
    def resolution(self):
        for track in self._mediainfo.tracks:
            if track.track_type == "Video":
                if track.height <= 480:
                    return "480p"
                elif track.height <= 576:
                    return "576p"
                elif track.height <= 720:
                    return "720p"
                elif track.height <= 1080:
                    if getattr(track, "scan_type__store_method") == "InterleavedFields":
                        return "1080i"
                    return "1080p"
                elif track.height <= 2160:
                    return "2160p"
                elif track.height <= 4320:
                    return "4320p"
        return ""

    @property
    def area(self):
        area_map = {
            "中国大陆": "CN",
            "中国香港": "HK",
            "中国台湾": "TW",
            "美国": "US",
            "日本": "JP",
            "韩国": "KR",
            "印度": "IN",
            "法国": "FR",
            "意大利": "IT",
            "德国": "GE",
            "西班牙": "ES",
            "葡萄牙": "PT",
        }
        regions = self._ptgen.get("region", [])
        for area in area_map.keys():
            if area in regions:
                return area_map[area]
        return ""

    @property
    def movie_name(self):
        if self._ptgen.get("site") == "imdb":
            return self._ptgen.get("name", "")
        if self._ptgen.get("site") == "douban":
            return self._ptgen.get("aka", [""])[0]
        return ""

    @property
    def movie_aka_name(self):
        return ""

    @property
    def size(self):
        for track in self._mediainfo.tracks:
            if track.track_type == "General":
                return track.file_size
        return ""

    @property
    def tags(self):
        tags = {}
        for track in self._mediainfo.tracks:
            if track.track_type == "General":
                if track.audio_language_list and "Chinese" in track.audio_language_list:
                    tags["chinese_audio"] = True
                if track.text_language_list and "Chinese" in track.text_language_list:
                    tags["chinese_subtitle"] = True
        # TODO: hdr, hdr10_plus, dolby_vision, diy, cantonese_audio, false,dts_x, dolby_atoms
        return tags

    @property
    def other_tags(self):
        return []

    @property
    def comparisons(self):
        return []

    @property
    def easy_upload_torrent_info(self):
        return EasyUpload(plugin=self).torrent_info

    @property
    def auto_feed_info(self):
        return AutoFeed(plugin=self).info

    def upload(self):
        self._prepare()
        if self.easy_upload:
            torrent_info = self.easy_upload_torrent_info
            if self.trim_description:
                # 直接打印简介部分来绕过浏览器的链接长度限制
                torrent_info["description"] = ""
            logger.trace(f"torrent_info: {torrent_info}")
            link = f"{self.upload_url}#torrentInfo={quote(json.dumps(torrent_info))}"
            logger.trace(f"已生成自动上传链接：{link}")
            if self.trim_description:
                logger.info(f"种子描述：\n{self.description}")
            open_link(link, self.use_short_url)
        elif self.auto_feed:
            link = f"{self.upload_url}{quote(self.auto_feed_info, safe='#:/=@')}"
            # if self.trim_description:
            #     logger.info(f"种子描述：\n{self.description}")
            logger.trace(f"已生成自动上传链接：{link}")
            open_link(link, self.use_short_url)
        else:
            logger.info(
                "\n"
                f"标题: {self.title}\n"
                f"副标题: {self.subtitle}\n"
                f"豆瓣: {self.douban_url}\n"
                f"IMDB: {self.imdb_url}\n"
                f"视频编码: {self.video_codec} 音频编码: {self.audio_codec} 分辨率: {self.resolution}\n"
                f"描述:\n{self.description}"
            )



import argparse

from differential.plugins.nexusphp import NexusPHP


class PTerClub(NexusPHP):

    @classmethod
    def get_aliases(cls):
        return 'pter',

    @classmethod
    def get_help(cls):
        return 'PTerClub插件，适用于PTerClub'

    @classmethod
    def add_parser(cls, parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
        return super().add_parser(parser)

    def __init__(self, **kwargs):
        super().__init__(upload_url="https://pterclub.com/upload.php", **kwargs)



from abc import abstractmethod


class TorrnetBase:

    @property
    @abstractmethod
    def title(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def subtitle(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def media_info(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def media_infos(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def description(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def original_description(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def douban_url(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def douban_info(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def imdb_url(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def screenshots(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def poster(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def year(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def category(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def video_type(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def format(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def source(self):
        raise NotImplementedError()
        return ""

    @property
    @abstractmethod
    def video_codec(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def audio_codec(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def resolution(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def area(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def movie_name(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def movie_aka_name(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def size(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def tags(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def other_tags(self):
        raise NotImplementedError()

    @property
    @abstractmethod
    def comparisons(self):
        raise NotImplementedError()



import re

from loguru import logger

from differential.version import version
from differential.utils.config import merge_config
from differential.plugins.base import PARSER, REGISTERED_PLUGINS


@logger.catch
def main():
    args = PARSER.parse_args()
    logger.info("Differential 差速器 {}".format(version))
    config = merge_config(args)

    if 'log' in config:
        log = config.pop('log')
        logger.add(log, level="TRACE", backtrace=True, diagnose=True)

    if hasattr(args, 'plugin'):
        plugin = config.pop('plugin')
        try:
            logger.trace(config)
            REGISTERED_PLUGINS[plugin](**config).upload()
        except TypeError as e:
            m = re.search(r'missing \d+ required positional argument[s]{0,1}: (.*?)$', str(e))
            if m:
                logger.error("缺少插件必需的参数，请检查输入的参数: {}".format(m.groups()[0]))
                return
            raise e
    else:
        PARSER.print_help()


if __name__ == '__main__':
    main()
